#   Xmi_logger


__title__ = "xmi_logger"
__description__ = "An enhanced logger based on Loguru"
__version__ = "0.0.7"
__author__ = "gm.zhibo.wang"
__author_email__ = "gm.zhibo.wang@gmail.com"
__url__ = "https://github.com/wang-zhibo/xmi_logger"
__license__ = "MIT"